//Program to monitor a kegerator.

//This program monitors:
//Four force sensors indicating the fullness of the keg
//A temperature sensor indicating the temperature of the keg
//A switch indicating the position of the tap handle
//
//This program reports these measurements to a web page
//and a twitter account.
//
//Created by Devlin Thyne at SparkFun Electronics
//http://www.sparkfun.com
//
// ----------------------------------------------------------------------------
//"THE BEER-WARE LICENSE" (Revision 42):
//<sparkfun.com> wrote this file. As long as you retain this notice you
//can do whatever you want with this stuff. If we meet some day, and you think
//this stuff is worth it, you can buy me a beer in return.
//--Devlin Thyne
//----------------------------------------------------------------------------

#include <WString.h>

#include <Ethernet.h>

#include <EEPROM.h>

#include <avr/pgmspace.h>

#include "password.h"

//Comment out the following line if the device is using a test network configuration
#define LIVE_NETWORK_CONFIG

/////////////////////////////////////////////////
//live network configuration, use your own values, gateway and subnet are optional.
/////////////////////////////////////////////////
byte aryLiveMac[] = { 0x66, 0x66, 0x66, 0x66, 0x66, 0x66 };
byte aryLiveIp[] =      { 192, 168, 1, 96 };
byte aryLiveGateway[] = { 192, 168, 1,  1 };
byte aryLiveSubnet[] =  { 255, 255, 0,  0 };

/////////////////////////////////////////////////
//test network configuration, use your own values, gateway and subnet are optional.
/////////////////////////////////////////////////
byte aryTestMac[] = {  0x77, 0x77, 0x77, 0x77, 0x77, 0x77 };
byte aryTestIp[] =      { 192, 168, 1, 95 };
byte aryTestGateway[] = { 192, 168, 1,  1 };
byte aryTestSubnet[] =  { 255, 255, 0,  0 };

/////////////////////////////////////////////////
//Twitter's IP address
/////////////////////////////////////////////////
byte aryTwitterAddress[] = { 168, 143, 162, 100 };




/////////////////////////////////////////////////
//Constants
/////////////////////////////////////////////////
const int cintEmptyForce = 1200;
const int cintFullForce = 1347;

//Define which port to use
const int cintPort = 80;

//Pin Declarations
const int cintTemperaturePin = 4;
const int cintHandleSensorPin = 19;
const int cintForcePin1 = 0;
const int cintForcePin2 = 1;
const int cintForcePin3 = 2;
const int cintForcePin4 = 3;

//The address of the pull count in EEPROM
//this is the most significant byte
const int cintPullCountLocation = 128;

/////////////////////////////////////////////////
//Global Variables
/////////////////////////////////////////////////
boolean boolTwittered = false;

String objTwitterStatus;


//Start the server class
Server objServer = Server(cintPort);


/////////////////////////////////////////////////
//Initialization
/////////////////////////////////////////////////
void setup()
{
  // initialize the ethernet device
#ifdef LIVE_NETWORK_CONFIG
  Ethernet.begin(aryLiveMac, aryLiveIp, aryLiveGateway, aryLiveSubnet);
#else
  Ethernet.begin(aryTestMac, aryTestIp, aryTestGateway, aryTestSubnet);
#endif

  //Set analog channel 5 as an input for the reed switched handle sensor
  pinMode(cintHandleSensorPin, INPUT);

  // start listening for clients
  objServer.begin();

}


/////////////////////////////////////////////////
//Main Loop
/////////////////////////////////////////////////
void loop()
{

  //Read temperature
  int intRawTemperature = analogRead(cintTemperaturePin);

  //Calculate temperature
  int intTemperature = intRawTemperature - 79;

  boolean boolHandlePulled = digitalRead(cintHandleSensorPin);

  //Read force sensors
  int intForce1 = analogRead(cintForcePin1);
  int intForce2 = analogRead(cintForcePin2);  
  int intForce3 = analogRead(cintForcePin3);
  int intForce4 = analogRead(cintForcePin4);

  //Add up the forces
  int intForceSum = intForce1 + intForce2 + intForce3 + intForce4;
  
  //Calculate the percentage full
  int intPercentFull = int(float(100.0 * (intForceSum - cintEmptyForce) / ( cintFullForce - cintEmptyForce )));


  //Form the tweet as a String
  objTwitterStatus = "Temp: ";
  objTwitterStatus.append(intTemperature);
  objTwitterStatus.append('\xb0');
  objTwitterStatus.append(" F.  Est ");
  objTwitterStatus.append(intPercentFull);
  objTwitterStatus.append("\x25 full");
  objTwitterStatus.append(" Pull Count: ");
  objTwitterStatus.append(functionGetPullCount());

  //If the handle is pulled and we have not twittered yet, then twitter
  if ( ( true == boolHandlePulled )  && ( false == boolTwittered ) )
  {
    //If we are not testing, then it is OK to twitter
    #ifdef LIVE_NETWORK_CONFIG
    functionTwitter(objTwitterStatus.getChars());
    #endif
    
    functionIncrementPullCount();
    boolTwittered = true;
  }
  
  //If the handle is not pulled and we have already twittered, then we are ready to twitter for the next pull
  if ( ( false == boolHandlePulled ) && ( true == boolTwittered ) )
  {
    boolTwittered = false;
  }
  

  //Get a client
  Client objClient = objServer.available();


  //If there is a cleint connecting, then send the web page
  if (true == objClient)
  {

    //Send the header information
    objServer.println("HTTP/1.0 200 OK");
    objServer.println("Server: arduino");
    objServer.println("Cache-Control: no-store, no-cache");
    objServer.println("Pragma: no-cache\nConnection: close");
    objServer.println("Content-Type: text/html\n");

    //Start sending the HTML page
    objServer.print("<html>");
    //The page header that tells the browser to refresh ever 60 seconds
    objServer.print("<head><meta http-equiv=\"refresh\" content=\"60\"><title>Sparkfun Keg Monitor</title></head>");
    objServer.print("<h1>");

    //The SparkFun logo
    objServer.print("<img src=\"http://static.sparkfun.com/images/framework/logo.gif\" />");

    objServer.print("</h1>");

    objServer.print("<p>Temperature: ");
    objServer.print(intTemperature, DEC);
    objServer.print("&deg;F</p>");
    
    objServer.print("Handle Pulled: ");

    if( true == boolHandlePulled )
    {  
      objServer.print("Yes");
    }
    else
    {
      objServer.print("No");
    }

    objServer.print("</p>");

    objServer.print("<p>Pull Count: ");
    objServer.print(functionGetPullCount(), DEC);
    objServer.print("</p>");


    objServer.print("<p>Estimated Percent Left: ");
    objServer.print(intPercentFull, DEC);
    objServer.print("%</p>");

    objServer.print("<p>Total Weight: ");
    objServer.print(intForceSum, DEC);
    objServer.print(" footons (not calibrated yet)</p>");
    objServer.print("Less than ");
    objServer.print(cintEmptyForce, DEC);
    objServer.print(" footons is empty</p>");

    objServer.print("<p>The Raw Data:</p>");

    objServer.print("<p>Temperature Reading: ");
    objServer.print(intRawTemperature, DEC);
    objServer.print("</p>");

    objServer.print("<p>Force 1: ");
    objServer.print(intForce1, DEC);
    objServer.print("</p>");

    objServer.print("<p>Force 2: ");
    objServer.print(intForce2, DEC);
    objServer.print("</p>");

    objServer.print("<p>Force 3: ");
    objServer.print(intForce3, DEC);
    objServer.print("</p>");

    objServer.print("<p>Force 4: ");
    objServer.print(intForce4, DEC);
    objServer.print("</p>");
    
    objServer.print("Twitter Status: ");
    objServer.print(objTwitterStatus.getChars());
    objServer.print("</p>");
    

    objServer.println("</html>");

    //Give a short delay and then stop the connection
    delay(1);
    objClient.stop();
  }
}

/////////////////////////////////////////////////
//Twitter Update
/////////////////////////////////////////////////
//This function uses code  from the following sites:
//http://pratham.name/post/39551949/twitter-php-script-without-curl
//http://www.arduino.cc/cgi-bin/yabb2/YaBB.pl?num=1234028200/1
//http://www.mats-vanselow.de/2009/02/08/arduino-lernt-twittern/
void functionTwitter(char* strTweet)
{

  //Start the connection to Twitter
  Client objTwitterClient(aryTwitterAddress, 80);


  if (objTwitterClient.connect())
  {
    //Start forming the POST data
    objTwitterClient.println("POST http://twitter.com/statuses/update.json HTTP/1.1");
    objTwitterClient.println("Host: twitter.com");

    //Send the authorization string
    objTwitterClient.print("Authorization: Basic ");
    objTwitterClient.println(strTwitterAuthorization);

    objTwitterClient.println("Content-type: application/x-www-form-urlencoded");

    //Send the calculated length of the status
    objTwitterClient.print("Content-length: ");
    objTwitterClient.println(9 + strlen(strTweet), DEC );

    objTwitterClient.println("Connection: Close");
    objTwitterClient.println();

    //Send the status
    objTwitterClient.print("status=");
    objTwitterClient.println(strTweet);


  }

  //Stop the Twitter client
  objTwitterClient.stop();

  delay(250);

}

/////////////////////////////////////////////////
//Read the pull count out of EEPROM
//Made with the help of:
//http://www.openobject.org/opensourceurbanism/Storing_Data
/////////////////////////////////////////////////
long functionGetPullCount()
{
  unsigned long ulongTemp = 0;

  //Read the individual bytes from EEPROM
  byte byte0 = EEPROM.read(cintPullCountLocation + 0);
  byte byte1 = EEPROM.read(cintPullCountLocation + 1);
  byte byte2 = EEPROM.read(cintPullCountLocation + 2);
  byte byte3 = EEPROM.read(cintPullCountLocation + 3);

  //Build up ulongTemp with the individual bytes
  //by casting the byte to a long and then rotating the long by
  //the appropriate number of bits for the byte's position
  //and then masking the result so that it will only change
  //the proper byte
  ulongTemp += ((long(byte3) << 24) & 0xff000000);
  ulongTemp += ((long(byte2) << 16) & 0x00ff0000);
  ulongTemp += ((long(byte1) << 8) & 0x0000ff00);
  ulongTemp += ((long(byte0) << 0) & 0x000000ff);

  return(ulongTemp);

}
/////////////////////////////////////////////////
//Increment the pull count in the EEPROM
//To delete the EEPROM, load and run the eeprom_clear sketch.
//Made with the help of:
//http://www.openobject.org/opensourceurbanism/Storing_Data
/////////////////////////////////////////////////
void functionIncrementPullCount()
{
  //Get the pull count, ulongTemp
  unsigned long ulongTemp = functionGetPullCount();
  
  //Increment the pull count, ulongTemp
  ulongTemp++;
  
  //Break up ulongTemp into individual bytes
  //by rotating ulongTemp the appropriate number of bits
  //and masking to only get the least significant byte  
  byte byte0 = ((ulongTemp >> 0) & 0xff);
  byte byte1 = ((ulongTemp >> 8) & 0xff);
  byte byte2 = ((ulongTemp >> 16) & 0xff);
  byte byte3 = ((ulongTemp >> 24) & 0xff);

  //Write the individual bytes to the EEPROM
  EEPROM.write(cintPullCountLocation + 0, byte0);
  EEPROM.write(cintPullCountLocation + 1, byte1);
  EEPROM.write(cintPullCountLocation + 2, byte2);
  EEPROM.write(cintPullCountLocation + 3, byte3);

}
